"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var immer_1 = tslib_1.__importStar(require("immer"));
var shallowequal_1 = tslib_1.__importDefault(require("shallowequal"));
/**
 * Context.
 */
var Context = /** @class */ (function () {
    function Context() {
        var _this = this;
        /**
         * listeners.
         */
        this.changed = [];
        /**
         * Consumer's select props.
         */
        this.selects = new Map();
        /**
         * set state.
         */
        this.setState = function (state) {
            _this.state = state;
        };
        /**
         * get state.
         */
        this.getState = function () {
            return _this.state;
        };
        /**
         * update state.
         */
        this.updateState = function (update) {
            // avoid produce when recursive call.
            if (_this.draft) {
                update(_this.draft);
                return;
            }
            // produce.
            _this.state = immer_1.default(_this.state, function (state) {
                _this.draft = state;
                update(state);
                _this.draft = undefined;
            }, _this.patchListener);
            // calculate changes selector.
            var changed = Array.from(_this.selects.entries()).reduce(function (changed, _a) {
                var select = _a[0], info = _a[1];
                var next = select(_this.state);
                if (!shallowequal_1.default(next, info.state)) {
                    info.state = next;
                    info.version++;
                    return true;
                }
                return changed;
            }, false);
            // notify if state changes.
            if (changed) {
                _this.changed.forEach(function (changed) { return changed(_this.state); });
            }
        };
        /**
         * apply patches.
         */
        this.applyPatches = function (patches) {
            _this.updateState(function (state) {
                immer_1.applyPatches(state, patches);
            });
        };
    }
    /**
     * subscribe state change.
     */
    Context.prototype.subscribe = function (changed) {
        this.changed.push(changed);
    };
    /**
     * unsubscribe state change.
     */
    Context.prototype.unsubscribe = function (changed) {
        var idx = this.changed.indexOf(changed);
        if (idx > -1) {
            this.changed.splice(idx, 1);
        }
    };
    /**
     * register select function.
     */
    Context.prototype.register = function (select) {
        this.selects.set(select, {
            state: select(this.state),
            version: 1
        });
    };
    /**
     * getSelectState.
     */
    Context.prototype.getSelectState = function (select) {
        return this.selects.get(select);
    };
    return Context;
}());
exports.Context = Context;
//# sourceMappingURL=Context.js.map